@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 
@@ -24,16 +24,125 @@
 #define NEEDS_BABL_DB
 #include "babl-internal.h"
 #include "babl-db.h"
+#include "babl-ref-pixels.h"
 
-static Babl *
-conversion_new (const char    *name,
-                int            id,
-                Babl          *source,
-                Babl          *destination,
-                BablFuncLinear linear,
-                BablFuncPlane  plane,
-                BablFuncPlanar planar,
-                void          *user_data)
+static void
+babl_conversion_plane_process (BablConversion *conversion,
+                               const void     *source,
+                               void           *destination,
+                               int             src_pitch,
+                               int             dst_pitch,
+                               long            n,
+                               void           *user_data)
+{
+  conversion->function.plane ((void*)conversion, source, destination,
+                              src_pitch, dst_pitch,
+                              n,
+                              user_data);
+}
+
+static void
+babl_conversion_planar_process (const Babl *babl,
+                                const char *src,
+                                char       *dst,
+                                long        n,
+                                void       *user_data)
+{
+  BablConversion *conversion = (void*)babl;
+  const BablImage *source = (void*)src;
+  BablImage       *destination = (void*)dst;
+#ifdef USE_ALLOCA
+  const char **src_data = alloca (sizeof (void *) * source->components);
+  char **dst_data = alloca (sizeof (void *) * destination->components);
+#else
+  const char  *src_data[BABL_MAX_COMPONENTS];
+  char  *dst_data[BABL_MAX_COMPONENTS];
+#endif
+
+  memcpy (src_data, source->data, sizeof (void *) * source->components);
+  memcpy (dst_data, destination->data, sizeof (void *) * destination->components);
+  conversion->function.planar ((void*)conversion,
+                                      source->components,
+                                      src_data,
+                                      source->pitch,
+                                      destination->components,
+                                      dst_data,
+                                      destination->pitch,
+                                      n,
+                                      user_data);
+}
+
+static void dispatch_plane (const Babl *babl,
+                            const char *source,
+                            char       *destination,
+                            long        n,
+                            void       *user_data)
+{
+  const BablConversion *conversion = &babl->conversion;
+  const void *src_data  = NULL;
+  void *dst_data  = NULL;
+  int   src_pitch = 0;
+  int   dst_pitch = 0;
+
+  if (BABL_IS_BABL (source))
+    {
+      BablImage *img;
+
+      img       = (BablImage *) source;
+      src_data  = img->data[0];
+      src_pitch = img->pitch[0];
+    }
+  if (BABL_IS_BABL (destination))
+    {
+      BablImage *img = (BablImage *) destination;
+
+      dst_data  = img->data[0];
+      dst_pitch = img->pitch[0];
+    }
+
+  if (!src_data)
+    src_data = source;
+  if (!src_pitch)
+    src_pitch = BABL (conversion->source)->type.bits / 8;
+  if (!dst_data)
+    dst_data = destination;
+  if (!dst_pitch)
+    dst_pitch = BABL (conversion->destination)->type.bits / 8;
+
+  babl_conversion_plane_process ((void*)conversion,
+                                 src_data, dst_data,
+                                 src_pitch, dst_pitch,
+                                 n, user_data);
+}
+
+static inline void
+babl_conversion_rig_dispatch (const Babl *babl)
+{
+  BablConversion *conversion = (BablConversion *) babl;
+  switch (BABL (conversion)->class_type)
+  {
+    case BABL_CONVERSION_PLANE:
+      conversion->dispatch = dispatch_plane;
+      break;
+    case BABL_CONVERSION_PLANAR:
+      conversion->dispatch = babl_conversion_planar_process;
+      break;
+    case BABL_CONVERSION_LINEAR:
+      conversion->dispatch = conversion->function.linear;
+      break;
+  }
+}
+
+Babl *
+_conversion_new (const char    *name,
+                 int            id,
+                 const Babl    *source,
+                 const Babl    *destination,
+                 BablFuncLinear linear,
+                 BablFuncPlane  plane,
+                 BablFuncPlanar planar,
+                 void          *user_data,
+                 int            allow_collision)
 {
   Babl *babl = NULL;
 
@@ -42,7 +151,11 @@ conversion_new (const char    *name,
 
   babl                = babl_malloc (sizeof (BablConversion) + strlen (name) + 1);
   babl->instance.name = (char *) babl + sizeof (BablConversion);
+#ifndef _WIN64
   strcpy (babl->instance.name, name);
+#else
+  strcpy_s (babl->instance.name, strlen(name) + 1, name);
+#endif
 
   if (linear)
     {
@@ -102,7 +215,6 @@ conversion_new (const char    *name,
   babl->conversion.cost        = 69L;
 
   babl->conversion.pixels      = 0;
-  babl->conversion.processings = 0;
 
   babl->conversion.data = user_data;
 
@@ -111,44 +223,44 @@ conversion_new (const char    *name,
     {
       const Babl *src_format = NULL;
       const Babl *dst_format = NULL;
-      if (BABL (babl->conversion.source) == babl_model_from_id (BABL_RGBA))
-        {
-          src_format = babl_format_from_id (BABL_RGBA_DOUBLE);
-          dst_format = babl_format_with_model_as_type (
-            BABL (babl->conversion.destination),
-            babl_type_from_id (BABL_DOUBLE));
-        }
-      else if (BABL (babl->conversion.destination) == babl_model_from_id (BABL_RGBA))
-        {
-          src_format = babl_format_with_model_as_type (
-            BABL (babl->conversion.source),
-            babl_type_from_id (BABL_DOUBLE));
-          dst_format = babl_format_from_id (BABL_RGBA_DOUBLE);
-        }
-      else
-        {
-          babl_fatal ("neither source nor destination model is RGBA (requirement might be temporary)");
-        }
+
+      src_format = babl_format_with_model_as_type (
+        BABL (babl->conversion.source),
+        babl_type_from_id (BABL_DOUBLE));
+      dst_format = babl_format_with_model_as_type (
+        BABL (babl->conversion.destination),
+        babl_type_from_id (BABL_DOUBLE));
+
+      if(allow_collision){
+        const Babl *fish = babl_conversion_find (src_format, dst_format);
+        if (fish)
+          return (void*)fish;
+      }
       babl_conversion_new (
         src_format,
         dst_format,
         "linear", linear,
         "data", user_data,
+        allow_collision?"allow-collision":NULL,
         NULL);
       babl->conversion.error = 0.0;
     }
 
+  babl_conversion_rig_dispatch (babl);
   return babl;
 }
 
 static char buf[512] = "";
 static int collisions = 0;
+
 static char *
-create_name (Babl *source, Babl *destination, int type)
+create_name (Babl *source, 
+             Babl *destination, 
+             int   type)
 {
   if (babl_extender ())
     {
-      snprintf (buf, 512 - 1, "%s %i: %s%s to %s",
+      snprintf (buf, sizeof (buf), "%s %i: %s%s to %s",
                 BABL (babl_extender ())->instance.name,
                 collisions,
                 type == BABL_CONVERSION_LINEAR ? "" :
@@ -156,21 +268,48 @@ create_name (Babl *source, Babl *destination, int type)
                 type == BABL_CONVERSION_PLANAR ? "planar " : "Eeeek! ",
                 source->instance.name,
                 destination->instance.name);
-      buf[511] = '\0';
     }
   else
     {
-      snprintf (buf, 512 - 1, "%s %s to %s",
+      snprintf (buf, sizeof (buf), "%s %s to %s %i",
                 type == BABL_CONVERSION_LINEAR ? "" :
                 type == BABL_CONVERSION_PLANE ? "plane " :
                 type == BABL_CONVERSION_PLANAR ? "planar " : "Eeeek! ",
                 source->instance.name,
-                destination->instance.name);
-      buf[511] = '\0';
+                destination->instance.name,
+                collisions);
     }
   return buf;
 }
 
+const char *
+babl_conversion_create_name (Babl *source, 
+                             Babl *destination, 
+                             int   type,
+                             int   allow_collision)
+{
+  Babl *babl;
+  char *name;
+  int id = 0;
+  collisions = 0;
+  name = create_name (source, destination, type);
+
+  if (allow_collision == 0)
+  {
+  babl = babl_db_exist (db, id, name);
+  while (babl)
+    {
+      /* we allow multiple conversions to be registered per extender, each
+         of them ending up with their own unique name
+       */
+      collisions++;
+      name = create_name (source, destination, type);
+      babl = babl_db_exist (db, id, name);
+    }
+  }
+  return name;
+}
+
 const Babl *
 babl_conversion_new (const void *first_arg,
                      ...)
@@ -189,8 +328,8 @@ babl_conversion_new (const void *first_arg,
 
   Babl          *source;
   Babl          *destination;
-
   char          *name;
+  int            allow_collision = 0;
 
   va_start (varg, first_arg);
   source      = (Babl *) arg;
@@ -213,6 +352,10 @@ babl_conversion_new (const void *first_arg,
           user_data = va_arg (varg, void*);
         }
 
+      else if (!strcmp (arg, "allow-collision"))
+        {
+          allow_collision = 1;
+        }
       else if (!strcmp (arg, "linear"))
         {
           if (got_func++)
@@ -266,22 +409,10 @@ babl_conversion_new (const void *first_arg,
       type = BABL_CONVERSION_PLANAR;
     }
 
-  collisions = 0;
-  name = create_name (source, destination, type);
+  name = (void*) babl_conversion_create_name (source, destination, type, allow_collision);
 
-  babl = babl_db_exist (db, id, name);
-  while (babl)
-    {
-      /* we allow multiple conversions to be registered per extender, each
-         of them ending up with their own unique name
-       */
-      collisions++;
-      name = create_name (source, destination, type);
-      babl = babl_db_exist (db, id, name);
-    }
-
-  babl = conversion_new (name, id, source, destination, linear, plane, planar, 
-                         user_data);
+  babl = _conversion_new (name, id, source, destination, linear, plane, planar,
+                          user_data, allow_collision);
 
   /* Since there is not an already registered instance by the required
    * id/name, inserting newly created class into database.
@@ -293,163 +424,6 @@ babl_conversion_new (const void *first_arg,
   return babl;
 }
 
-static long
-babl_conversion_linear_process (BablConversion *conversion,
-                                const void     *source,
-                                void           *destination,
-                                long            n)
-{
-  return conversion->function.linear (source, destination, n, conversion->data);
-}
-
-static long
-babl_conversion_plane_process (BablConversion *conversion,
-                               const void     *source,
-                               void           *destination,
-                               int             src_pitch,
-                               int             dst_pitch,
-                               long            n)
-{
-  return conversion->function.plane (source, destination,
-                                     src_pitch, dst_pitch,
-                                     n,
-                                     conversion->data);
-}
-
-static long
-babl_conversion_planar_process (BablConversion *conversion,
-                                BablImage      *source,
-                                BablImage      *destination,
-                                long            n)
-{
-#ifdef USE_ALLOCA
-  const char **src_data = alloca (sizeof (void *) * source->components);
-  char **dst_data = alloca (sizeof (void *) * destination->components);
-#else
-  const char  *src_data[BABL_MAX_COMPONENTS];
-  char  *dst_data[BABL_MAX_COMPONENTS];
-#endif
-
-  memcpy (src_data, source->data, sizeof (void *) * source->components);
-  memcpy (dst_data, destination->data, sizeof (void *) * destination->components);
-
-  return conversion->function.planar (source->components,
-                                      src_data,
-                                      source->pitch,
-                                      destination->components,
-                                      dst_data,
-                                      destination->pitch,
-                                      n,
-                                      conversion->data);
-}
-
-long
-babl_conversion_process (const Babl *babl,
-                         const char *source,
-                         char       *destination,
-                         long        n)
-{
-  BablConversion *conversion = (BablConversion *) babl;
-
-  babl_assert (BABL_IS_BABL (conversion));
-
-  switch (BABL (conversion)->class_type)
-    {
-      case BABL_CONVERSION_PLANE:
-      {
-        const void *src_data  = NULL;
-        void *dst_data  = NULL;
-        int   src_pitch = 0;
-        int   dst_pitch = 0;
-
-        if (BABL_IS_BABL (source))
-          {
-            BablImage *img;
-
-            img       = (BablImage *) source;
-            src_data  = img->data[0];
-            src_pitch = img->pitch[0];
-          }
-        if (BABL_IS_BABL (destination))
-          {
-            BablImage *img = (BablImage *) destination;
-
-            dst_data  = img->data[0];
-            dst_pitch = img->pitch[0];
-          }
-
-        if (!src_data)
-          src_data = source;
-        if (!src_pitch)
-          src_pitch = BABL (conversion->source)->type.bits / 8;
-        if (!dst_data)
-          dst_data = destination;
-        if (!dst_pitch)
-          dst_pitch = BABL (conversion->destination)->type.bits / 8;
-
-        babl_conversion_plane_process (conversion,
-                                       src_data, dst_data,
-                                       src_pitch, dst_pitch,
-                                       n);
-      }
-        break;
-
-      case BABL_CONVERSION_PLANAR:
-        babl_assert (BABL_IS_BABL (source));
-        babl_assert (BABL_IS_BABL (destination));
-
-        babl_conversion_planar_process (conversion,
-                                        (BablImage *) source,
-                                        (BablImage *) destination,
-                                        n);
-        break;
-
-      case BABL_CONVERSION_LINEAR:
-        /* the assertions relied on a babl_malloc structure
-         *
-         * babl_assert (!BABL_IS_BABL (source));
-           babl_assert (!BABL_IS_BABL (destination));*/
-
-        babl_conversion_linear_process (conversion,
-                                        source,
-                                        destination,
-                                        n);
-        break;
-
-      default:
-        babl_log ("args=(%s, %p, %p, %li) unhandled conversion type: %s",
-                  conversion->instance.name, source, destination, n,
-                  babl_class_name (conversion->instance.class_type));
-        return 0;
-        break;
-    }
-
-  conversion->processings++;
-  conversion->pixels += n;
-  return n;
-}
-
-#define test_pixels    128
-
-
-static double *
-test_create (void)
-{
-  static double test[sizeof (double) * test_pixels * 4];
-  int           i;
-  static int done = 0;
-
-  if (done)
-    return test;
-
-  srandom (20050728);
-
-  for (i = 0; i < test_pixels * 4; i++)
-    test [i] = (double) random () / RAND_MAX;
-
-  done = 1;
-  return test;
-}
 
 long
 babl_conversion_cost (BablConversion *conversion)
@@ -467,19 +441,15 @@ babl_conversion_error (BablConversion *conversion)
   Babl *fmt_source;
   Babl *fmt_destination;
 
-  const Babl *fmt_rgba_double = babl_format_new (babl_model ("RGBA"),
-                                                 babl_type ("double"),
-                                                 babl_component ("R"),
-                                                 babl_component ("G"),
-                                                 babl_component ("B"),
-                                                 babl_component ("A"),
-                                                 NULL);
-
+  const Babl *fmt_rgba_double = babl_format_with_space ("RGBA double",
+                                                 conversion->destination->format.space);
   double  error       = 0.0;
   long    ticks_start = 0;
   long    ticks_end   = 0;
 
-  double *test;
+  const int test_pixels = babl_get_num_conversion_test_pixels ();
+  const double *test = babl_get_conversion_test_pixels ();
+
   void   *source;
   void   *destination;
   double *destination_rgba_double;
@@ -518,13 +488,12 @@ babl_conversion_error (BablConversion *conversion)
         fmt_source->class_type == BABL_FORMAT &&
         fmt_destination->class_type == BABL_FORMAT))
     {
-      conversion->error = 0.000042;
+      conversion->error = 0.0000042;
     }
 
-  test = test_create ();
-
-
-  source                      = babl_calloc (test_pixels, fmt_source->format.bytes_per_pixel);
+  source                      = babl_calloc (test_pixels+1, fmt_source->format.bytes_per_pixel);
+                                                 /* +1 is masking valgrind Invalid read of 16
+                                                  *    false positives */
   destination                 = babl_calloc (test_pixels, fmt_destination->format.bytes_per_pixel);
   ref_destination             = babl_calloc (test_pixels, fmt_destination->format.bytes_per_pixel);
   destination_rgba_double     = babl_calloc (test_pixels, fmt_rgba_double->format.bytes_per_pixel);
@@ -533,10 +502,19 @@ babl_conversion_error (BablConversion *conversion)
   babl_process (fish_rgba_to_source,
                 test, source, test_pixels);
 
-  ticks_start = babl_ticks ();
-  babl_process (babl_fish_simple (conversion),
-                source, destination, test_pixels);
-  ticks_end = babl_ticks ();
+  if (BABL(conversion)->class_type == BABL_CONVERSION_LINEAR)
+  {
+    ticks_start = babl_ticks ();
+    babl_process (babl_fish_simple (conversion),
+                  source, destination, test_pixels);
+    ticks_end = babl_ticks ();
+  }
+  else
+  {
+    /* we could still measure it, but for the paths we only really consider
+     * the linear ones anyways */
+    ticks_end = 1000;
+  }
 
   babl_process (fish_reference,
                 source, ref_destination, test_pixels);
@@ -550,10 +528,6 @@ babl_conversion_error (BablConversion *conversion)
                               ref_destination_rgba_double,
                               test_pixels * 4);
 
-  fish_rgba_to_source->fish.processings--;
-  fish_reference->fish.processings--;
-  fish_destination_to_rgba->fish.processings -= 2;
-
   fish_rgba_to_source->fish.pixels      -= test_pixels;
   fish_reference->fish.pixels           -= test_pixels;
   fish_destination_to_rgba->fish.pixels -= 2 * test_pixels;
@@ -565,9 +539,23 @@ babl_conversion_error (BablConversion *conversion)
   babl_free (ref_destination_rgba_double);
 
   conversion->error = error;
-  conversion->cost  = babl_process_cost (ticks_start, ticks_end);
+  conversion->cost  = ticks_end - ticks_start;
 
   return error;
 }
 
+const Babl *
+babl_conversion_get_source_space (const Babl *conversion)
+{
+  return conversion->conversion.source->format.space;
+}
+
+const Babl *
+babl_conversion_get_destination_space (const Babl *conversion)
+{
+  return conversion->conversion.destination->format.space;
+}
+
+
+
 BABL_CLASS_IMPLEMENT (conversion)
