@@ -43,6 +43,11 @@ typedef enum
   BABL_CPU_ACCEL_X86_BMI1    = 0x00008000,
   BABL_CPU_ACCEL_X86_BMI2    = 0x00004000,
   BABL_CPU_ACCEL_X86_AVX2    = 0x00002000,
+  BABL_CPU_ACCEL_X86_AVX512F = 0x00001000,
+  BABL_CPU_ACCEL_X86_AVX512DQ= 0x00000800,
+  BABL_CPU_ACCEL_X86_AVX512CD= 0x00000400,
+  BABL_CPU_ACCEL_X86_AVX512BW= 0x00000200,
+  BABL_CPU_ACCEL_X86_AVX512VL= 0x00000100,
 
   BABL_CPU_ACCEL_X86_64_V2 =
     (BABL_CPU_ACCEL_X86_POPCNT|
@@ -61,6 +66,14 @@ typedef enum
      BABL_CPU_ACCEL_X86_OSXSAVE|
      BABL_CPU_ACCEL_X86_MOVBE),
 
+  BABL_CPU_ACCEL_X86_64_V4 =
+    (BABL_CPU_ACCEL_X86_64_V3|
+     BABL_CPU_ACCEL_X86_AVX512F|
+     BABL_CPU_ACCEL_X86_AVX512DQ|
+     BABL_CPU_ACCEL_X86_AVX512CD|
+     BABL_CPU_ACCEL_X86_AVX512BW|
+     BABL_CPU_ACCEL_X86_AVX512VL),
+
   /* powerpc accelerations */
   BABL_CPU_ACCEL_PPC_ALTIVEC = 0x00000010,
 
