@@ -64,7 +64,11 @@ extension_new (const char *path,
   babl                = babl_malloc (sizeof (BablExtension) + strlen (path) + 1);
   babl_set_destructor (babl, babl_extension_destroy);
   babl->instance.name = (char *) babl + sizeof (BablExtension);
+#ifndef _WIN64
   strcpy (babl->instance.name, path);
+#else
+  strcpy_s (babl->instance.name, strlen(path) + 1, path);
+#endif
   babl->instance.id         = 0;
   babl->class_type          = BABL_EXTENSION;
   babl->extension.dl_handle = dl_handle;
@@ -126,7 +130,9 @@ babl_extension_deinit (void)
 
 #ifdef BABL_DYNAMIC_EXTENSIONS
 
+#ifndef _WIN32
 #include <dirent.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #ifndef _WIN32
@@ -319,13 +325,21 @@ expand_path (char *path)
 
   src = path;
 
+#if defined(_WIN32) || defined(_WIN64)
+  return babl_strdup (path);
+#endif
+
   while (*src)
     {
-      char *home;
+      char *home = NULL;
       switch (*src)
         {
           case '~':
+#ifndef _WIN64
             home = getenv ("HOME");
+#else
+            _dupenv_s (&home, NULL, "HOME");
+#endif
             if (NULL != home)
               dst = babl_strcat (dst, home);
             break;
